#!/usr/bin/env python

from qpid.connection import Connection
from qpid.datatypes import Message, uuid4
from qpid.util import connect
import time
import sys
import simplejson
import pycurl
import tempfile
import pprint
import fabulous.image
import urllib

import optparse
parser = optparse.OptionParser()
parser.add_option("-t", "--targets", dest="targets", default="localhost",
                  help="comma-separated list of target hostnames running qpid")
parser.add_option("-d", "--debug", dest="debug", action="store_true",
                  help="debug what messages are being sent")
options, args = parser.parse_args()

options.targets = [t.strip() for t in options.targets.split(',')]

# Create connection and session
session_dicts = []
for target in options.targets:
    print "Attempting to setup connection with", target
    try:
        socket = connect(target, 5672)
        connection = Connection(
            socket, username='guest', password='guest',
        )
        connection.start(timeout=10000)
        session = connection.session(str(uuid4()))

        # Setup routing properties
        properties = session.delivery_properties(routing_key='http_latlon')
        session_dicts.append({
            'target': target,
            'socket': socket,
            'connection': connection,
            'session': session,
            'properties': properties,
        })
        print "    Created target", target
    except Exception as e:
        print "    Failed to create target", target
        print str(e)
        import traceback
        traceback.print_exc()

if len(session_dicts) == 0:
    print "No connections.  Dieing early."
    time.sleep(5)  # 5 seconds.
    sys.exit()

hashdict = {}
top5 = {}
def on_receive(data):
    global top5
    global hashdict
    obj = simplejson.loads(data)

    if 'entities' in obj:
        if 'hashtags' in obj['entities']:
            for tag in obj['entities']['hashtags']:
                hashdict[tag['text']] = hashdict.get(tag['text'], 0) + 1

#    if hashdict:
#        tuples = [(k, v) for k, v in hashdict.iteritems()]
#        tuples.sort(lambda a, b : -1 * cmp(a[1], b[1]))
#        dtuple = dict(tuples[:5])
#
#        if dtuple != top5:
#            pprint.pprint(dtuple)
#            top5 = dtuple

    try:
        lnk = obj['user']['profile_image_url']
        local_file = tempfile.mkstemp()[1]
        urllib.urlretrieve(lnk, local_file)
        print fabulous.image.Image(local_file)
    except Exception as e:
        print str(e)

    #pprint.pprint(obj)
    if not 'geo' in obj or not obj['geo']:
        return

    lat, lon = obj['geo']['coordinates']
    msg = simplejson.dumps({
        'lat': lat,
        'lon': lon,
    })

    if options.debug:
        print "[sending]", msg

    for d in session_dicts:
        d['session'].message_transfer(
            destination='amq.topic',
            message=Message(d['properties'], msg))

STREAM_URL = "https://stream.twitter.com/1/statuses/sample.json"
USER = "DevAccount7"
PASS = "blahah"

conn = pycurl.Curl()
conn.setopt(pycurl.USERPWD, "%s:%s" % (USER, PASS))
conn.setopt(pycurl.URL, STREAM_URL)
conn.setopt(pycurl.WRITEFUNCTION, on_receive)

print "Entering mainloop"
print "Sending to", ",".join([d['target'] for d in session_dicts])

try:
    conn.perform()
except Exception as e:
    print str(e)
finally:
    # Close session
    for d in session_dicts:
        d['session'].close(timeout=10)



#!/usr/bin/env python

import simplejson
import pycurl
import tempfile
#import fabulous.image
import fabulous.text
import fabulous.color
import urllib
import os
import re

def main():
    of_interest = [
        {
            'attr': 'user_mentions',
            'symb': '@',
            'color': fabulous.color.green,
            'extract': 'screen_name',
        },
        {
            'attr': 'hashtags',
            'symb': '#',
            'color': fabulous.color.red,
            'extract': 'text',
        },
        {
            'attr': 'urls',
            'symb': '',
            'color': fabulous.color.yellow,
            'extract': 'url',
        },
    ]

    def on_receive(data):
        obj = simplejson.loads(data)

        try:
            if not 'user' in obj:
                return
            lnk = obj['user']['profile_image_url']
            tstamp = ' '.join(obj['created_at'].split(' ')[:4])
            tstamp = fabulous.color.magenta(tstamp)
            local_file = tempfile.mkstemp()[1]
            urllib.urlretrieve(lnk, local_file)
            #print fabulous.image.Image(local_file)
            os.remove(local_file)
            if 'text' in obj:
                try:
                    for lookup in of_interest:
                        things = obj['entities'][lookup['attr']]
                        things = [thing[lookup['extract']] for thing in things]

                        for thing in things:
                            item = lookup['symb'] + thing
                            toks = unicode(obj['text']).split(item)
                            obj['text'] = toks[0] + lookup['color'](item) + toks[1]

                except Exception as e:
                    import traceback
                    traceback.print_exc()
                    print obj['entities']
                    print fabulous.color.cyan("color failed..."), str(e)

                print tstamp,
                print unicode(obj['text'])

        except Exception as e:
            import traceback
            traceback.print_exc()
            print str(e), ':('


    STREAM_URL = "https://stream.twitter.com/1/statuses/sample.json"
    USER = "DevAccount7"
    PASS = "blahah"

    conn = pycurl.Curl()
    conn.setopt(pycurl.USERPWD, "%s:%s" % (USER, PASS))
    conn.setopt(pycurl.URL, STREAM_URL)
#FIXME: sometimes it just hangs... lets find out why and fix it someday
#conn.setopt(pycurl.SOCKET_TIMEOUT, 12)
    conn.setopt(pycurl.WRITEFUNCTION, on_receive)
    conn.perform()



#!/usr/bin/env python


try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

f = open('README.rst')
long_description = f.read().strip()
f.close()

setup(
    name='twabulous',
    version='9000.2a',
    description="Twitter + Fabulous = Twabulous",
    long_description=long_description,
    author='Ralph Bean & Remy DeCausemaker',
    author_email='ralph.bean@gmail.com',
    url='http://github.com/ralphbean/twabulous/',
    license='AGPLv3',
    install_requires=[
        "fabulous",
        "simplejson",
        "pycurl",
        "PIL",
    ],
    packages=['twabulous'],
    include_package_data=True,
    zip_safe=False,
    entry_points="""
    [console_scripts]
    twabulous = twabulous.twabulous:main
    """
)
